# --------------------------------------------------------------------------------- #
# WINDOWTREE wxPython IMPLEMENTATION
# Inspired By And Heavily Based On WindowTree By Carl Godkin.
#
# Andrea Gavana, @ 03 Ma4 2007
# Latest Revision: 03 Mar 2007, 22.30 CET
#
# For All Kind Of Problems, Requests Of Enhancements And Bug Reports, Please
# Write To Me At:
#
# gavana@kpo.kz
# andrea.gavana@gmail.com
#
# Or, Obviously, To The wxPython Mailing List!!!
#
#
# End Of Comments
# --------------------------------------------------------------------------------- #

"""
Description
===========

WindowTree can easily be uses to look at the Sizer and/or Window hierarchy
of any wxPython applications. Since I have some quite dynamic interfaces,
it's handy to inspect them using the tree views that WindowTree provides. 


Features
========

The main features are Sizer or Window hierarchy laid out using a
wx.TreeCtrl. I have used a variety of icons to indicate the window
state or the sizer type.  (See Help->Legend) When you select a node in
the window tree (i.e., a wxWindow), its size is displayed.  When you
select a node in the sizer tree (i.e., a wxWindow or wxSizer) in sizer
hierarchy mode, the size and most sizer options are displayed.  If the
Tree->Flash selected windows toggle is set, a box outlining the window
or sizer is flashed when a node is selected.  To help test a layout,
there is a popup menu on each tree node that calls Hide, Show, Enable,
Disable, etc.. Others could be added of course.
There is also an alternative (more descriptive) icon set, which associates
to every known control a particular icon. Please toggle the "Edit" =>
"Use Full Descriptive Icons" menu to enable it.


Usage
=====

If you want to try it out, you might add something like this in your
initialization (either main() or OnInit):

# Change it to 0 if you don't want the WindowTree
SHOW_WINDOWTREE = 1

def main():
    app = wx.PySimpleApp()
    demoFrame = WindowTreeDemo(None, -1, "WindowTree wxPython Demo :-D", size=(600, 400))
    demoFrame.CenterOnScreen()
    demoFrame.Show()

    if SHOW_WINDOWTREE:
        windowtree = WindowTreeFrame(None, -1)
        demoFrame.windowtree = windowtree
        windowtree.Show()
        
    app.MainLoop()


Supported Platforms
===================

WindowTree has been tested on the following platforms:
  * Windows (Windows XP);


Latest Revision: Andrea Gavana @ 03 Mar 2007, 22.30 CET
Version 0.1

"""


import wx
import os
import glob

ID_WINDOW_SHOW = wx.ID_HIGHEST + 10001
ID_SIZER_SHOW = ID_WINDOW_SHOW + 1
ID_WINDOW_ENABLE = ID_WINDOW_SHOW + 2

NUM_IMAGES = 13

#----------------------------------------------------------------------------
# Provides these bitmaps:
#  0: Solid gray   (IsShown, !IsEnabled)
#  1: Solid blue   (IsShown, IsEnabled)
#  2: Outline gray (!IsShown, !IsEnabled)
#  3: Outline blue (!IsShown, IsEnabled)
#  4: Vertical sizer
#  5: Horizontal sizer
#  6: Notebook sizer
#  7: Grid sizer
#  8: Flex grid sizer
#  9: Other sizer
# 10: Spacer
#----------------------------------------------------------------------------


def GetBitmap(index):

    if index == 0:
    
        xpm_data = ["12 10 2 1","  c None","a c #C0C0C0","            ","            ",
                    "  aaaaaaaa  ","  aaaaaaaa  ","  aaaaaaaa  ","  aaaaaaaa  ",
                    "  aaaaaaaa  ","  aaaaaaaa  ","            ","            "]
        
    elif index == 1:
    
        xpm_data = ["12 10 2 1","  c None","a c #0000FF","            ","            ",
                    "  aaaaaaaa  ","  aaaaaaaa  ","  aaaaaaaa  ","  aaaaaaaa  ","  aaaaaaaa  ",
                    "  aaaaaaaa  ","            ","            "]
        
    elif index == 2:
    
        xpm_data = ["12 10 2 1","  c None","a c #C0C0C0","            ","            ",
                    "  aaaaaaaa  ","  a      a  ","  a      a  ","  a      a  ","  a      a  ",
                    "  aaaaaaaa  ","            ","            "]
        
    elif index == 3:
    
        xpm_data = ["12 10 2 1","  c None","a c #0000FF","            ","            ",
                    "  aaaaaaaa  ","  a      a  ","  a      a  ","  a      a  ","  a      a  ",
                    "  aaaaaaaa  ","            ","            "]
    
    elif index == 4:

        xpm_data = ["12 10 2 1","  c None","a c #FF0000","            ","  aaaaaaaa  ",
                    "  a      a  ","  a      a  ","  aaaaaaaa  ","  a      a  ","  a      a  ",
                    "  a      a  ","  aaaaaaaa  ","            "]
            
    elif index == 5:

        xpm_data = ["12 10 2 1","  c None","a c #FF0000","            ","            ",
                    " aaaaaaaaaa "," a   a    a "," a   a    a "," a   a    a "," a   a    a ",
                    " aaaaaaaaaa ","            ","            "]
        
    elif index == 6:
    
        xpm_data = ["12 10 3 1","  c None","a c Black","b c #FF0000","    a a     ",
                    "  b aaaa b  ","  b  a a b  ","  b      b  ","  bbbbbbbb  ","  b      b  ",
                    "  b      b  ","  b      b  ","  bbbbbbbb  ","            "]
            
    elif index == 7:

        xpm_data = ["12 10 3 1","  c None","a c Black","b c #FF0000","            ",
                    "   a a      "," b aaaa bbb "," b  a a   b "," b   b    b "," b   b    b ",
                    " b   b    b "," bbbbbbbbbb ","            ","            "]
        
    elif index == 8:

        xpm_data = ["12 10 2 1","  c None","a c #FF0000","            ","  aa aa     ",
                    " a  a  a    "," a  aaaaaa  "," a        a "," a        a "," a        a ",
                    " a        a "," aaaaaaaaaa ","            "]

    elif index == 9:
    
        xpm_data = ["12 10 2 1","  c None","a c #FF0000","            "," aaaaaaaaa  ",
                    " a   a   a  "," a   a   a  "," aaaaaaaaa  "," a   a   a  "," a   a   a  ",
                    " aaaaaaaaa  ","            ","            "]
            
    elif index == 10:
    
        xpm_data = ["12 10 2 1","  c None","a c #FF0000","            "," aaaaaaaaaa ",
                    " a     a  a "," a     a  a "," a     a  a "," aaaaaaaaaa "," a     a  a ",
                    " a     a  a "," aaaaaaaaaa ","            "]
        
    elif index == 11:
    
        xpm_data = ["12 10 2 1","  c None","a c #FF0000","            "," aaaaaaaaaa ",
                    " a        a "," a        a "," a        a "," a        a "," a        a ",
                    " a        a "," aaaaaaaaaa ","            "]
            
    elif index == 12:
    
        xpm_data = ["12 10 2 1","  c None","a c Black","            ","            ",
                    "  aaaaaaaa  ","  a      a  ","  a      a  ","  a      a  ","  a      a  ",
                    "  aaaaaaaa  ","            ","            "]

    else:

        return wx.NullBitmap

    return wx.BitmapFromXPMData(xpm_data)


class WindowTreeItemData(object):
    """ Class used to store information about a window, a sizer or a spacer. """

    def __init__(self, inputWidget, label="", option=-1, flag=-1, border=-1):
        """ Default class constructor. """

        self._window = self._sizer = self._spacer = False
        
        if isinstance(inputWidget, wx.Window):
            self._window = inputWidget
        elif isinstance(inputWidget, wx.Sizer):
            self._sizer = inputWidget
        else:
            self._spacer = inputWidget
            self._width, self._height = inputWidget
            
        self._option = option
        self._flag = flag
        self._border = border
        self._label = label
        

    def isWindow(self):
        """ Returns whether the item is a widget or not. """

        return self._window


    def isSizer(self):
        """ Returns whether the item is a sizer or not. """

        return self._sizer

    
    def isSpacer(self):
        """ Returns whether the item is a spacer or not. """

        return self._spacer


    def layout(self):
        """ Layouts the sizer (if the item is a sizer). """

        if self._sizer:
            self._sizer.Layout()
            

    def fit(self):
        """ Fits the window (if the item is a widget). """

        if self._window:
            self._window.Fit()
            

    def show(self, show=True):
        """ Shows/hides the window (if the item is a widget). """

        if self._window:
            self._window.Show(show)
            

    def enable(self, enable=True):
        """ Enable/disables the window (if the item is a widget). """

        if self._window:
            self._window.Enable(enable)
            

    def refresh(self):
        """ Refreshes the window (if the item is a widget). """

        if self._window:
            self._window.Refresh()
            

    def flash(self):
        """ Flashes the selected widget/sizer using wx.ScreenDC. """

        if self._window:
            x1 = y1 = 0
            x1, y1 = self._window.ClientToScreen((x1, y1))
            size = self._window.GetSize()

            self.drawOnDC(size, x1, y1)

        elif self._sizer:
            pos = self._sizer.GetPosition()
            x1 = pos.x
            y1 = pos.y
            x1, y1 = self.findSizerParentWindow(self._sizer).ClientToScreen((x1, y1))
            size = self._sizer.GetSize()

            self.drawOnDC(size, x1, y1)


    def drawOnDC(self, size, x1, y1):
        """ Actually draws on the wx.ScreenDC. """

        dc = wx.ScreenDC()
        x2 = x1 + size.GetWidth()
        y2 = y1 + size.GetHeight()
        dc.SetLogicalFunction(wx.INVERT)
        dc.DrawLine(x1, y1, x2, y1)
        dc.DrawLine(x2, y1, x2, y2)
        dc.DrawLine(x2, y2, x1, y2)
        dc.DrawLine(x1, y2, x1, y1)

    
    def sizerShow(self, show=True):
        """ Shows/hides the sizer and its content. """

        if self._window:
            containingSizer = self._window.GetContainingSizer()
            if containingSizer:
                containingSizer.Show(self._window, show)
                containingSizer.Layout()

        elif self._sizer:
            containingSizer = self.getContainingSizer(self._sizer)
            if containingSizer:
                containingSizer.Show(self._sizer, show)
                containingSizer.Layout()
    
  
#----------------------------------------------------------------------------
# There is no direct connection between a sizer and its
# containing sizer(if any).  So find it by finding first
# the sizer's parent window.  This window must have a 
# sizer and it will be the containing sizer or will
# contain the containing sizer.
#----------------------------------------------------------------------------

    def getContainingSizer(self, szr):
        """ Utility method. """

        parent = self.findSizerParentWindow(szr)
        containingSizer = None
        if parent and parent.GetSizer():
            containingSizer = self.findParentSizer(parent.GetSizer(), szr)

        return containingSizer

#----------------------------------------------------------------------------
# Check ancestor and ancestor's descendent sizers to find
# the sizer containing szr, i.e., the parent sizer for szr.
#----------------------------------------------------------------------------

    def findParentSizer(self, ancestor, szr):
        """ Utility method. """

        children = ancestor.GetChildren()
        
        for child in children:
            if child.IsSizer():
                if szr == child.GetSizer():
                    return ancestor

            # Recurse.
            s = self.findParentSizer(child.GetSizer(), szr)
            if s:
                return s

        return None


    def findSizerParentWindow(self, szr):
        """ Utility method. """

        children = szr.GetChildren()
        
        for child in children:
            if child.IsWindow():
                return child.GetWindow().GetParent()

            elif child.IsSizer():
              # Recurse.
              return self.findSizerParentWindow(child.GetSizer())
  
        return None


    def info(self, frame):
        """ Logs information about widgets and sizers. """

        self._size = wx.Size()
        
        if self._sizer:
            self._size = self._sizer.GetSize()
        elif self._window:
            self._size = self._window.GetSize()
        else:
            self._size.SetWidth(self._width)
            self._size.SetHeight(self._height)

        if self._option == -1:
            wx.LogMessage("Name: %s\nSize: %d x %d\n"%(self._label, self._size.GetWidth(), self._size.GetHeight()))
        else:
            wx.LogMessage("Name: %s\nSize: %d x %d\nProportion: %d\nFlag: %s\nBorder: %d\n"%
                          (self._label, self._size.GetWidth(), self._size.GetHeight(), self._option,
                           self.expandFlag(self._flag), self._border))

        
    def expandFlag(self, flag):
        """ Returns a string representation of the sizer's flags. """

        count = 0
        s = ""

        if flag & wx.EXPAND:
            s += "wx.EXPAND"
            count += 1

        if flag & wx.SHAPED:
            if count:
                s += "|"
            s += "wx.SHAPED"
            count += 1

        if flag & wx.ADJUST_MINSIZE:
            if count:
                s += "|"
            s += "wx.ADJUST_MINSIZE"
            count += 1

        if flag & wx.ALIGN_LEFT:
            if count:
                s += "|"
            s += "wx.ALIGN_LEFT"
            count += 1

        if flag & wx.ALIGN_TOP:
            if count:
                s += "|"
            s += "wx.ALIGN_TOP"
            count += 1

        if flag & wx.ALIGN_RIGHT:
            if count:
                s += "|"
            s += "wx.ALIGN_RIGHT"
            count += 1

        if flag & wx.ALIGN_BOTTOM:
            if count:
                s += "|" 
            s += "wx.ALIGN_BOTTOM"
            count += 1

        # Would only have one of these
        if flag & wx.ALIGN_CENTER:
            if count:
                s += "|"
            s += "wx.ALIGN_CENTER"
            count += 1

        elif flag & wx.ALIGN_CENTER_HORIZONTAL:
            if count:
                s += "|"
            s += "wx.ALIGN_CENTER_HORIZONTAL"
            count += 1

        elif flag & wx.ALIGN_CENTER_VERTICAL:
            if count:
                s += "|"
            s += "wx.ALIGN_CENTER_VERTICAL"
            count += 1

        # If we have wxALL, then don't need to check individual borders
        if flag & wx.ALL:
            if count:
                s += "|"
            s += "wx.ALL"
            count += 1

        else:
            if flag & wx.TOP:
                if count:
                    s += "|"
                s += "wx.TOP"
                count += 1
            if flag & wx.LEFT:
                if count:
                    s += "|"
                s += "wx.LEFT"
                count += 1
            if flag & wx.RIGHT:
                if count:
                    s += "|"
                s += "wx.RIGHT"
                count += 1
            if flag & wx.BOTTOM:
                if count:
                    s += "|"
                s += "wx.BOTTOM"
                count += 1

        # Some other unrecognized flag
        if count == 0:
            s = "%d"%flag

        return s


class WindowTreeFrame(wx.Frame):
    """ The main application frame. """
    
    def __init__(self, parent, id=wx.ID_ANY, title="Window Tree", pos=wx.DefaultPosition,
                 size=(250, 500), style=wx.DEFAULT_FRAME_STYLE|wx.FRAME_NO_TASKBAR|wx.STAY_ON_TOP):
        """ Default class constructor. """
        
        wx.Frame.__init__(self, parent, id, title, pos, size, style)

        # Make a panel.
        panel = wx.Panel(self, -1)
        sizer = wx.BoxSizer(wx.VERTICAL)

        # Put the tree in the panel
        self._tree = WindowTree(panel, -1)
        sizer.Add(self._tree, 1, wx.EXPAND)
        panel.SetSizerAndFit(sizer)

        self.CreateStatusBar(1)
        self.BuildMenuBar()

        self.SetPosition((5, 5))


    def BuildMenuBar(self):
        """ Builds the frame's menubar. """

        menuBar = wx.MenuBar()

        menu = wx.Menu()
        menuItem = wx.MenuItem(menu, -1, "&Close\tCtrl+Q", "Close WindowTree")
        menu.AppendItem(menuItem)
        self.Bind(wx.EVT_MENU, self.OnClose, menuItem)

        menuBar.Append(menu, "&File")

        menu = wx.Menu()
        menuItem = wx.MenuItem(menu, -1, "Update wx&Windows Hierarchy \tCtrl+W")
        menu.AppendItem(menuItem)
        self.Bind(wx.EVT_MENU, self.OnUpdateWindowHierarchy, menuItem)
        menuItem = wx.MenuItem(menu, -1, "Update wx&Sizers Hierarchy \tCtrl+S")
        menu.AppendItem(menuItem)
        self.Bind(wx.EVT_MENU, self.OnUpdateSizerHierarchy, menuItem)
        
        menu.AppendSeparator()

        menuItem = wx.MenuItem(menu, -1, "Flash selected Windows", kind=wx.ITEM_CHECK)
        menu.AppendItem(menuItem)
        self.Bind(wx.EVT_MENU, self.OnDoFlash, menuItem)

        menuItem = wx.MenuItem(menu, -1, "Use Full Descriptive Icons", kind=wx.ITEM_CHECK)
        menu.AppendItem(menuItem)
        self.Bind(wx.EVT_MENU, self.OnUseFullIcons, menuItem)
        
        menuBar.Append(menu, "&Tree")

        menu = wx.Menu()
        menuItem = wx.MenuItem(menu, -1, "&Legend")
        menu.AppendItem(menuItem)
        self.Bind(wx.EVT_MENU, self.OnLegend, menuItem)
        
        menuBar.Append(menu, "&Help")

        self.SetMenuBar(menuBar)


    def OnClose(self, event):
        """ Handles the wx.EVT_CLOSE event for WindowTreeFrame. """

        self.Destroy()        


    def OnUpdateWindowHierarchy(self, event):
        """ Updates the window hierarchy from the menu. """

        self._tree.Freeze()
        self._tree.updateWindowHierarchy()

        root = self._tree.GetRootItem()
        if root.IsOk():
            self._tree.expandAll(root)

        self._tree.Thaw()


    def OnUpdateSizerHierarchy(self, event):
        """ Updates the sizer hierarchy from the menu. """

        self._tree.Freeze()
        self._tree.updateSizerHierarchy()

        root = self._tree.GetRootItem()
        if root.IsOk():
            self._tree.expandAll(root)

        self._tree.Thaw()


    def OnDoFlash(self, event):

        self._tree.setFlash(event.IsChecked())


    def OnUseFullIcons(self, event):

        self._tree.setUseFullIcons(event.IsChecked())
        

    def OnLegend(self, event):

        dialog = WindowTreeLegend(self)
        dialog.CenterOnScreen()
        dialog.Show()


class WindowTreeLegend(wx.Dialog):

    def __init__(self, parent):

        wx.Dialog.__init__(self, parent, -1, "Window Tree Legend")

        labels = ["wxWindow(IsShown, !IsEnabled)", # 0
                  "wxWindow(IsShown, IsEnabled)", # 1
                  "wxWindow(!IsShown, !IsEnabled)", # 2
                  "wxWindow(!IsShown, IsEnabled)", # 3
                  "Vertical wxBoxSizer",	# 4
                  "Horizontal wxBoxSizer",	# 5
                  "Vertical wxStaticBoxSizer", # 6
                  "Horizontal wxStaticBoxSizer", # 7
                  "wxNotebookSizer",		# 8
                  "wxGridSizer",		# 9
                  "wxFlexGridSizer",		# 10
                  "Other sizer",		# 11
                  "Spacer",			# 12
                  ]
  

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(wx.StaticLine(self, -1), 0, wx.EXPAND|wx.ALL, 10)
        fsizer = wx.FlexGridSizer(0, 2, 5, 5)

        for i in xrange(NUM_IMAGES):
            fsizer.Add(wx.StaticBitmap(self, -1, GetBitmap(i)),
                       1, wx.LEFT|wx.ALIGN_CENTER_VERTICAL, 5)
            fsizer.Add(wx.StaticText(self, -1, labels[i]))
    
        sizer.Add(fsizer)
        sizer.Add(wx.StaticLine(self, -1), 0, wx.EXPAND|wx.LEFT|wx.RIGHT|wx.TOP, 10)
        sizer.Add(self.CreateButtonSizer(wx.OK), 0, wx.CENTRE|wx.ALL, 10)

        self.SetSizerAndFit(sizer)


class WindowTree(wx.TreeCtrl):
    """ The wx.TreeCtrl used to display the window/sizer hierarchy. """

    def __init__(self, parent, id=wx.ID_ANY, pos=wx.DefaultPosition,
                 size=wx.DefaultSize, style=0):
        """ Default class constructor. """
        
        wx.TreeCtrl.__init__(self, parent, id, pos, size, style|wx.TR_HIDE_ROOT)

        self._flashCount = 0
        self._flasher = wx.TreeItemId()
        self._timer = None
        self._enableFlash = False
        self._useFullIcons = False
        self._filenames = []
        self._lastAction = 0

        self.CreateImageList()
        self.CreatePopupMenu()
        self.BindEvents()


    def CreateImageList(self):
        """ Creates the wx.TreeCtrl image list. """
        
        if self._useFullIcons:
            ilist = wx.ImageList(16, 16)
            images = glob.glob("images/*.png") + glob.glob("images/*.xpm")
            for img in images:
                ilist.Add(wx.Bitmap(img, wx.BITMAP_TYPE_ANY))
                self._filenames.append(os.path.split(os.path.splitext(img)[0])[1])
        else:
            ilist = wx.ImageList(12, 10)
            for i in xrange(NUM_IMAGES):
                ilist.Add(GetBitmap(i))

        self.AssignImageList(ilist)


    def CreatePopupMenu(self):
        """ Creates the wx.TreeCtrl item popup menu. """
        
        # Create popup menus for tree
        sizerPopup = wx.Menu()
        
        menuItem = wx.MenuItem(sizerPopup, -1, "sizer.Layout()")
        sizerPopup.AppendItem(menuItem)
        self.Bind(wx.EVT_MENU, self.OnSizerLayout, menuItem)

        sizerPopup.AppendSeparator()

        menuItem = wx.MenuItem(sizerPopup, ID_SIZER_SHOW, "containing_sizer.Show(sizer)")
        sizerPopup.AppendItem(menuItem)
        self.Bind(wx.EVT_MENU, self.OnSizerShow, menuItem)

        menuItem = wx.MenuItem(sizerPopup, -1, "containing_sizer.Hide(sizer)")
        sizerPopup.AppendItem(menuItem)
        self.Bind(wx.EVT_MENU, self.OnSizerShow, menuItem)

        windowPopup = wx.Menu()

        menuItem = wx.MenuItem(windowPopup, ID_WINDOW_ENABLE, "window.Enable()")
        windowPopup.AppendItem(menuItem)
        self.Bind(wx.EVT_MENU, self.OnWindowEnable, menuItem)

        menuItem = wx.MenuItem(windowPopup, -1, "window.Disable()")
        windowPopup.AppendItem(menuItem)
        self.Bind(wx.EVT_MENU, self.OnWindowEnable, menuItem)
        
        windowPopup.AppendSeparator()

        menuItem = wx.MenuItem(windowPopup, ID_WINDOW_SHOW, "window.Show()")
        windowPopup.AppendItem(menuItem)
        self.Bind(wx.EVT_MENU, self.OnWindowShow, menuItem)

        menuItem = wx.MenuItem(windowPopup, -1, "window.Hide()")
        windowPopup.AppendItem(menuItem)
        self.Bind(wx.EVT_MENU, self.OnWindowShow, menuItem)

        windowPopup.AppendSeparator()

        menuItem = wx.MenuItem(windowPopup, -1, "window.Refresh()")
        windowPopup.AppendItem(menuItem)
        self.Bind(wx.EVT_MENU, self.OnWindowRefresh, menuItem)

        menuItem = wx.MenuItem(windowPopup, -1, "window.Fit()")
        windowPopup.AppendItem(menuItem)
        self.Bind(wx.EVT_MENU, self.OnWindowFit, menuItem)

        windowPopup.AppendSeparator()

        menuItem = wx.MenuItem(windowPopup, ID_SIZER_SHOW, "containing_sizer.Show(window)")
        windowPopup.AppendItem(menuItem)
        self.Bind(wx.EVT_MENU, self.OnSizerShow, menuItem)

        menuItem = wx.MenuItem(windowPopup, -1, "containing_sizer.Hide(window)")
        windowPopup.AppendItem(menuItem)
        self.Bind(wx.EVT_MENU, self.OnSizerShow, menuItem)

        self._sizerPopup = sizerPopup
        self._windowPopup = windowPopup


    def BindEvents(self):
        """ Binds the wx.TreeCtrl events. """

        self.Bind(wx.EVT_TREE_SEL_CHANGED, self.OnSelection)
        self.Bind(wx.EVT_TIMER, self.OnFlash)
        self.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)
        

    def GetSuitableImage(self, win):
        """ Returns a suitable bitmap depending on the widget type. """

        found = 0
        winclassname = repr(win).split(";")[0]
        winclassname = winclassname.strip(">").strip("<")
        winclassname = winclassname.replace("_controls.", "").replace("_windows.", "")
        winclassname = winclassname.replace("_stc.", "").replace("_core.", "")
        winclassname = winclassname.replace("_animate.", "").replace("_wizard.", "")
        winclassname = winclassname.replace("_calendar.", "")
        winclassname = winclassname.replace("Py", "")

        if "__main__" in winclassname:
            winclassname = win.GetClassName()
            winclassname = winclassname[0:2] + "." + winclassname[2:]
        
        for count, files in enumerate(self._filenames):
            if winclassname == files: 
                found = 1
                break

        if found:
            return count

        return self.GetImageList().GetImageCount() - 2

                
    def updateWindowHierarchy(self):
        """ Updates the window hierarchy. """

        self._lastAction = 1
        
        # Delete all the node in the tree.
        self.DeleteAllItems()

        # Create a root node(which we're hiding anyhow)
        rootId = self.AddRoot("Root")

        # Loop over top level windows and build tree for each.
        for item in wx.GetTopLevelWindows():
            # Skip WindowTree frame.
            if isinstance(item, WindowTreeFrame):
                continue

            # Skip WindowTree legend dialog
            if isinstance(item, WindowTreeLegend):
                continue

            self.addWindowNodes(rootId, item)

        firstChild, cookie = self.GetFirstChild(rootId)
        wx.CallAfter(self.EnsureVisible, firstChild)
        

    def addWindowNodes(self, parent, win):
        """ Adds children nodes in the windows hierarchy. """

        name = win.GetName()
        label = win.GetClassName() + " " + name

        # Add title for frames and labels for wxControls
        if isinstance(win, wx.Frame):
            label += " (" + win.GetTitle() + ")"

        elif isinstance(win, wx.Control):
            text = win.GetLabel()
            if len(text) > 40:
                text = text[0:40] + "..."
            if text:
                label += " (" + text + ")"

        image = -1
        if win.IsShown():
            if win.IsEnabled():
                image = 1
            else:
                image = 0
        else:
            if win.IsEnabled():
                image = 3
            else:
                image = 2

        if self._useFullIcons:
            image = self.GetSuitableImage(win)
        
        data = WindowTreeItemData(win, label)
        leaf = self.AppendItem(parent, label, image, -1)
        self.SetPyData(leaf, data)

        children = win.GetChildren()

        for child in children:
            if not isinstance(child, wx.Frame):
                # Recurse.
                self.addWindowNodes(leaf, child)
  

    def updateSizerHierarchy(self):
        """ Updates the sizer hierarchy. """

        self._lastAction = 2
        
        # Delete all the node in the tree.
        self.DeleteAllItems()

        # Create a root node(which we're hiding anyhow)
        rootId = self.AddRoot("Root")

        # Loop over top level windows and build tree for each.
        for item in wx.GetTopLevelWindows():
            # Skip WindowTree frame.
            if isinstance(item, WindowTreeFrame):
                continue

            # Skip WindowTree legend dialog
            if isinstance(item, WindowTreeLegend):
                continue

            self.addSizerNodes(rootId, item)
  
        firstChild, cookie = self.GetFirstChild(rootId)
        wx.CallAfter(self.EnsureVisible, firstChild)

        
    def addSizerNodes(self, parent, win, opt=-1, flg=-1, bdr=-1):
        """ Adds children nodes in the sizers hierarchy. """

        if isinstance(win, wx.Sizer):
            self.addSizerNodes2(parent, win, opt, flg, bdr)
            return
        
        name = win.GetName()
        label = win.GetClassName() + " " + name

        # Add title for frames and labels for wxControls
        if isinstance(win, wx.Frame):
            label += " (" + win.GetTitle() + ")"

        elif isinstance(win, wx.Control):
            text = win.GetLabel()
            if len(text) > 40:
                text = text[0:40] + "..."
            if text:
                label += " (" + text + ")"

        image = -1        
        if win.IsShown():
            if win.IsEnabled():
                image = 1
            else:
                image = 0
        else:
            if win.IsEnabled():
                image = 3
            else:
                image = 2

        if self._useFullIcons:
            image = self.GetSuitableImage(win)

        data = WindowTreeItemData(win, label, opt, flg, bdr)
        leaf = self.AppendItem(parent, label, image, -1)
        self.SetPyData(leaf, data)

        sizer = win.GetSizer()

        if sizer:
            self.addSizerNodes(leaf, sizer)
        else:
            children = win.GetChildren()
            for child in children:
                if not isinstance(child, wx.Frame):
                    # Recurse.
                    self.addSizerNodes(leaf, child)


    def addSizerNodes2(self, parent, sizer, opt=-1, flg=-1, bdr=-1):
        """ Adds children nodes in the sizers hierarchy. """

        label = sizer.GetClassName()

        if isinstance(sizer, wx.StaticBoxSizer):
            if sizer.GetOrientation() == wx.VERTICAL:
                image = 6
                label += " Vertical"
            elif sizer.GetOrientation() == wx.HORIZONTAL:
                image = 7
                label += " Horizontal"
            else:
                # Should never happen
                image = 11

        elif isinstance(sizer, wx.BoxSizer):
            if sizer.GetOrientation() == wx.VERTICAL:
                image = 4
                label += " Vertical"
            elif sizer.GetOrientation() == wx.HORIZONTAL:
                image = 5
                label += " Horizontal"
            else:
                # Should never happen
                image = 11

        elif isinstance(sizer, wx.FlexGridSizer):
            image = 10		# Flex grid sizer

        elif isinstance(sizer, wx.GridSizer):
            image = 9		# Grid sizer

        else:
            image = 11		# Some other kind of sizer

        if self._useFullIcons:
            image = self.GetSuitableImage(sizer)
        
        # Add leaf to tree.
        data = WindowTreeItemData(sizer, label, opt, flg, bdr)
        leaf = self.AppendItem(parent, label, image, -1)
        self.SetPyData(leaf, data)
        
        # Special treatment for static box sizer.
        if isinstance(sizer, wx.StaticBoxSizer):
            self.addSizerNodes(leaf, sizer.GetStaticBox())

        children = sizer.GetChildren()
        
        for child in children:
            if child.IsWindow():
                # Recurse.
                self.addSizerNodes(leaf, child.GetWindow(), child.GetProportion(),
                                   child.GetFlag(), child.GetBorder())
            elif child.IsSizer():
                # Recurse.
                self.addSizerNodes(leaf, child.GetSizer(), child.GetProportion(),
                                   child.GetFlag(), child.GetBorder())
            elif child.IsSpacer():
                data = WindowTreeItemData(child.GetMinSize(), "Spacer", child.GetProportion(),
                                          child.GetFlag(), child.GetBorder())                
                item = self.AppendItem(leaf, "Spacer", self.GetImageList().GetImageCount()-1, -1)
                self.SetPyData(item, data)


    def expandAll(self, item):
        """ Expands all the children of item. """

        if item != self.GetRootItem() and not self.IsExpanded(item):
            self.Expand(item)

        child, cookie = self.GetFirstChild(item)
        while child.IsOk():
            self.expandAll(child)
            child, cookie = self.GetNextChild(item, cookie)


    def setFlash(self, flash):
        """ Sets whether the window/sizer should flash when selected. """

        self._enableFlash = flash


    def setUseFullIcons(self, useFullIcons):
        """ Sets whether the image list should show full descriptive icons or not. """

        self._useFullIcons = useFullIcons

        self.Freeze()
        
        # Delete all the node in the tree.
        self.DeleteAllItems()
        self.CreateImageList()

        if self._lastAction == 1:     # wxWindow Hierarchy
            self.updateWindowHierarchy()
        elif self._lastAction == 2:   # wxSizer Hierarchy
            self.updateSizerHierarchy()

        self.expandAll(self.GetRootItem())

        self.Thaw()


    def OnSelection(self, event):
        """ Handles the wx.EVT_TREE_SEL_CHANGED event fr WindowTree. """

        if self._timer:
            # If there's a timer running now, turn it off.
            self._timer.Stop()
            self._timer = None

        id = event.GetItem()
        if not id.IsOk():
            return

        data = self.GetPyData(id)
        if not data:
            # No tree data.  Give up
            return

        data.info(self.getFrame())

        if self._enableFlash and (data.isWindow() or data.isSizer()):
            data.flash()		# Highlight window or sizer

            self._flasher = id
            self._flashCount = 4		# Make this an even number so
                                        # that colors end up right again.
            self._timer = wx.Timer(self, -1)
            self._timer.Start(400, False)
  

    def getFrame(self):

        # Look for our frame.
        w = self.GetParent()
        while not w.IsTopLevel():
            w = w.GetParent()

        return w


    def OnFlash(self, event):
        """ Handles the wx.EVT_TIMER event fr WindowTree. """

        if not self._flasher.IsOk():
            return

        if self._flashCount < 0:
            self._timer.Stop()
            self._timer = None
        else:
            data = self.GetPyData(self._flasher)
            data.flash()
            self._flashCount -= 1
  

    def OnRightDown(self, event):
        """ Handles the wx.EVT_RIGHT_DOWN event fr WindowTree. """

        id, flags = self.HitTest(event.GetPosition())
        
        if id.IsOk():
            self.SelectItem(id)

        data = self.GetPyData(id)
        if not data:
            # No tree data.  Give up
            return

        data.info(self.getFrame())

        if data.isSizer(): 
            # Popup menu on sizers
            self.PopupMenu(self._sizerPopup, event.GetPosition())

        elif data.isWindow():
            # Popup menu on windows
            self.PopupMenu(self._windowPopup, event.GetPosition())


    def OnSizerLayout(self, event):
        """ Layouts the selected sizer. """

        id = self.GetSelection()
        data = self.GetPyData(id)
        data.layout()


    def OnSizerShow(self, event):
        """ Shows/hides the selected sizer and its children. """
        
        id = self.GetSelection()
        data = self.GetPyData(id)
        data.sizerShow(event.GetId() == ID_SIZER_SHOW)


    def OnWindowFit(self, event):
        """ Fits the selected window. """

        id = self.GetSelection()
        data = self.GetPyData(id)
        data.fit()


    def OnWindowShow(self, event):
        """ Shows/hides the selected window. """

        id = self.GetSelection()
        data = self.GetPyData(id)
        data.show(event.GetId() == ID_WINDOW_SHOW)


    def OnWindowEnable(self, event):
        """ Enables/disables the selected window. """

        id = self.GetSelection()
        data = self.GetPyData(id)
        data.enable(event.GetId() == ID_WINDOW_ENABLE)


    def OnWindowRefresh(self, event):
        """ Refreshes the selected window. """

        id = self.GetSelection()
        data = self.GetPyData(id)
        data.refresh()


_textCtrls = ["Welcome", "To", "The", "WindowTree", "Demo", "For", "wxPython",
              "Andrea Gavana", "03 March 2007"]

class MyLog(wx.PyLog):

    def __init__(self, textCtrl, logTime=0):

        wx.PyLog.__init__(self)
        self.tc = textCtrl
        self.logTime = logTime

    def DoLogString(self, message, timeStamp):
        if self.tc:
            self.tc.Freeze()
            self.tc.AppendText(message + '\n')
            self.tc.ScrollLines(message.count("\n")+2)
            self.tc.Thaw()


class WindowTreeDemo(wx.Frame):

    def __init__(self, parent, id=wx.ID_ANY, title="", pos=wx.DefaultPosition,
                 size=wx.DefaultSize, style=wx.DEFAULT_FRAME_STYLE):

        wx.Frame.__init__(self, parent, id, title, pos, size, style)

        self.panel = wx.Panel(self, -1)
        self.dummypanel1 = wx.ScrolledWindow(self.panel, -1)
        self.dummypanel2 = wx.PyPanel(self.panel, -1)

        self.CreateControls()
        self.LayoutControls()

        self.Bind(wx.EVT_CLOSE, self.OnClose)
        

    def CreateControls(self):
        
        # Add a bunch of buttons
        self.buttons = []
        for i in xrange(9):
            self.buttons.append(wx.Button(self.dummypanel1, -1, "Button %d"%(i+1)))

        # Add a bunch of checkboxes
        self.checks = []
        for i in xrange(9):
            self.checks.append(wx.CheckBox(self.dummypanel2, -1, "CheckBox %d"%(i+1)))

        # Add a bunch of textctrls:
        self.texts = []
        for i in xrange(9):
            self.texts.append(wx.TextCtrl(self.panel, -1, _textCtrls[i]))

        # Add the log window
        self.logText = wx.TextCtrl(self.panel, -1, "",
                                   style=wx.TE_RICH2 | wx.TE_MULTILINE | wx.TE_READONLY)
        wx.Log_SetActiveTarget(MyLog(self.logText))
    

    def LayoutControls(self):

        mainsizer = wx.BoxSizer(wx.VERTICAL)

        topHSizer = wx.BoxSizer(wx.HORIZONTAL)
        
        topLeftStaticBox = wx.StaticBox(self.dummypanel1, -1, "Bunch Of Buttons")
        topLeftSizer = wx.StaticBoxSizer(topLeftStaticBox, wx.VERTICAL)
        topLeftFlex = wx.FlexGridSizer(3, 3, 5, 5)

        topRightStaticBox = wx.StaticBox(self.dummypanel2, -1, "Bunch Of CheckBoxes")
        topRightSizer = wx.StaticBoxSizer(topRightStaticBox, wx.VERTICAL)
        topRightGrid = wx.GridSizer(3, 3, 5, 5)

        bottomVSizer = wx.BoxSizer(wx.VERTICAL)
        
        # Layout the buttons in a FlexGridSizer
        for btn in self.buttons:
            topLeftFlex.Add(btn, 0, wx.EXPAND | wx.ALIGN_CENTER)

        topLeftSizer.Add(topLeftFlex, 0, wx.EXPAND | wx.ALL, 5)
        
        # Layout the checkboxes in a GridSizer
        for chk in self.checks:
            topRightGrid.Add(chk, 0, wx.EXPAND | wx.ALIGN_CENTER)

        topRightSizer.Add(topRightGrid, 0, wx.EXPAND | wx.ALL, 5)

        self.dummypanel1.SetSizer(topLeftSizer)
        self.dummypanel2.SetSizer(topRightSizer)
        
        topHSizer.Add(self.dummypanel1, 0, wx.ALL, 5)
        topHSizer.Add(self.dummypanel2, 0, wx.ALL, 5)
        
        # Layout the textctrls using BoxSizers
        for indx, txts in enumerate(self.texts):
            if indx%3 == 0:  # Row changes
                textCtrlSizer = wx.BoxSizer(wx.HORIZONTAL)
            textCtrlSizer.Add(txts, 1, wx.EXPAND | wx.ALL, 5)
            if indx%3 == 2:  # Row ends, add to the bottom sizer
                bottomVSizer.Add(textCtrlSizer, 1, wx.EXPAND)
                bottomVSizer.AddSpacer((5, 2))

        mainsizer.Add(topHSizer, 1, wx.EXPAND)
        mainsizer.AddSpacer((10, 10))
        mainsizer.Add(bottomVSizer, 1, wx.EXPAND|wx.ALL, 5)
        mainsizer.AddSpacer((10, 10))
        mainsizer.Add(self.logText, 1, wx.EXPAND|wx.ALL, 5)
        self.panel.SetSizer(mainsizer)
        mainsizer.Layout()


    def OnClose(self, event):

        if hasattr(self, "windowtree"):
            try:
                self.windowtree.Destroy()
            except:
                pass

        self.Destroy()            


# Change it to 0 if you don't want the WindowTree
SHOW_WINDOWTREE = 1

def main():
    app = wx.PySimpleApp()
    demoFrame = WindowTreeDemo(None, -1, "WindowTree wxPython Demo :-D", size=(600, 400))
    demoFrame.CenterOnScreen()
    demoFrame.Show()

    if SHOW_WINDOWTREE:
        windowtree = WindowTreeFrame(None, -1)
        demoFrame.windowtree = windowtree
        windowtree.Show()
        
    app.MainLoop()


if __name__ == "__main__":
    main()

    
